@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -160,13 +160,15 @@
 #define Y_16_TO_Y_8(val,dst) dst=((val+0x80)>>8)
 #define UV_16_TO_UV_8(val,dst) dst=((val+0x80)>>8)
 #define YJ_16_TO_Y_8(val,dst)  dst=(((int)val * 219+0x8000)>>16) + 16;
-#define YJ_16_TO_Y_16(val,dst)  dst=(((int)val * 219+0x8000)>>8) + 0x1000;
+#define YJ_16_TO_Y_16(val,dst)  dst=(((int)val * 219+0x80)>>8) + 0x1000;
+#define UVJ_16_TO_UV_16(val,dst)  dst=(((int)val * 224+0x80)>>8) + 0x1000;
 
 #else
 #define Y_16_TO_Y_8(val,dst) dst=((val)>>8)
 #define UV_16_TO_UV_8(val,dst) dst=((val)>>8)
 #define YJ_16_TO_Y_8(val,dst)  dst=(((int)val * 219)>>16) + 16;
 #define YJ_16_TO_Y_16(val,dst)  dst=(((int)val * 219)>>8) + 0x1000;
+#define UVJ_16_TO_UV_16(val,dst)  dst=(((int)val * 224)>>8) + 0x1000;
 #endif
 
 
@@ -174,15 +176,15 @@
 #define Y_16_TO_YJ_8(val,dst)   dst=((((RECLIP(val, 0x1000, 0xEB00)-0x1000)*255)/219+0x80)>>8)
 #define UV_16_TO_UVJ_8(val,dst) dst=((((RECLIP(val, 0x1000, 0xF000)-0x1000)*255)/224+0x80)>>8)
 
-#define Y_16_TO_YJ_16(val,dst)   dst=(((uint32_t)((RECLIP((uint32_t)val, 0x1000, 0xEB00)-0x1000)*65535)/219+0x8000)>>16)
-#define UV_16_TO_UVJ_16(val,dst) dst=((((RECLIP(val, 0x1000, 0xF000)-0x1000)*65535)/224+0x8000)>>16)
+#define Y_16_TO_YJ_16(val,dst)   dst=(((uint32_t)((RECLIP((uint32_t)val, 0x1000, 0xEB00)-0x1000)*65535)/219+0x80)>>8)
+#define UV_16_TO_UVJ_16(val,dst) dst=(((uint32_t)((RECLIP((uint32_t)val, 0x1000, 0xF000)-0x1000)*65535)/224+0x80)>>8)
 
 #else
 #define Y_16_TO_YJ_8(val,dst)   dst=((((RECLIP(val, 0x1000, 0xEB00)-0x1000)*255)/219)>>8)
 #define UV_16_TO_UVJ_8(val,dst) dst=((((RECLIP(val, 0x1000, 0xF000)-0x1000)*255)/224)>>8)
 
 #define Y_16_TO_YJ_16(val,dst)   dst=((((uint32_t)(RECLIP(val, 0x1000, 0xEB00)-0x1000)*65535)/219)>>8)
-//#define UV_16_TO_UVJ_16(val,dst) dst=((((RECLIP(val, 0x1000, 0xF000)-0x1000)*65535)/224)>>16)
+#define UV_16_TO_UVJ_16(val,dst) dst=((((uint32_t)(RECLIP(val, 0x1000, 0xF000)-0x1000)*65535)/224)>>8)
 
 #endif
 
