@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -425,6 +425,295 @@ static void bswap_64_c(void * data, int len)
     }
   }
 
+#define GENERIC_CLIP(val, min, max) (val>max?max:(val<min?min:val))
+
+/* Add functions */
+
+static void add_u8_c(const void * _src1, const void * _src2,
+                     void * _dst, int num)
+  {
+  int32_t tmp;
+  int i = num+1;
+  const uint8_t * src1 = _src1;
+  const uint8_t * src2 = _src2;
+  uint8_t * dst  = _dst;
+  
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++);
+    *(dst++) = RECLIP_32_TO_8(tmp);
+    }
+  }
+
+static void add_s8_c(const void * _src1, const void * _src2,
+                     void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const int8_t * src1 = _src1;
+  const int8_t * src2 = _src2;
+  int8_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-128,127);
+    }
+  }
+
+static void add_u8_s_c(const void * _src1, const void * _src2,
+                       void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const uint8_t * src1 = _src1;
+  const uint8_t * src2 = _src2;
+  uint8_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++) - 256;
+    *(dst++) = GENERIC_CLIP(tmp,-128,127) + 128;
+    }
+  }
+
+
+static void add_u16_c(const void * _src1, const void * _src2,
+                      void * _dst, int num)
+  {
+  int32_t tmp;
+  int i = num+1;
+  const uint16_t * src1 = _src1;
+  const uint16_t * src2 = _src2;
+  uint16_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++);
+    *(dst++) = RECLIP_32_TO_16(tmp);
+    }
+
+  }
+
+static void add_s16_c(const void * _src1, const void * _src2,
+                      void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const int16_t * src1 = _src1;
+  const int16_t * src2 = _src2;
+  int16_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-32768,32767);
+    }
+  }
+
+static void add_u16_s_c(const void * _src1, const void * _src2,
+                        void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const uint16_t * src1 = _src1;
+  const uint16_t * src2 = _src2;
+  uint16_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++) - 65536;
+    *(dst++) = GENERIC_CLIP(tmp,-32768,32767) + 32768;
+    }
+  }
+
+static void add_s32_c(const void * _src1, const void * _src2,
+                      void * _dst, int num)
+  {
+  int64_t tmp;
+  int i = num+1;
+  const int32_t * src1 = _src1;
+  const int32_t * src2 = _src2;
+  int32_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-2147483648,2147483647);
+    }
+  }
+
+static void add_float_c(const void * _src1, const void * _src2,
+                        void * _dst, int num)
+  {
+  int i = num+1;
+  const float * src1 = _src1;
+  const float * src2 = _src2;
+  float * dst  = _dst;
+
+  while(--i)
+    {
+    *(dst++) = *(src1++) + *(src2++);
+    }
+  }
+
+static void add_double_c(const void * _src1, const void * _src2,
+                         void * _dst, int num)
+  {
+  int i = num+1;
+  const double * src1 = _src1;
+  const double * src2 = _src2;
+  double * dst  = _dst;
+  
+  while(--i)
+    {
+    *(dst++) = *(src1++) + *(src2++);
+    }
+  }
+
+/* Subtract functions */
+
+static void sub_u8_c(const void * _src1, const void * _src2,
+                     void * _dst, int num)
+  {
+  int32_t tmp;
+  int i = num+1;
+  const uint8_t * src1 = _src1;
+  const uint8_t * src2 = _src2;
+  uint8_t * dst  = _dst;
+  
+  while(--i)
+    {
+    tmp = *(src1++) - *(src2++);
+    *(dst++) = RECLIP_32_TO_8(tmp);
+    }
+  }
+
+static void sub_s8_c(const void * _src1, const void * _src2,
+                     void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const int8_t * src1 = _src1;
+  const int8_t * src2 = _src2;
+  int8_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) - *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-128,127);
+    }
+  }
+
+static void sub_u8_s_c(const void * _src1, const void * _src2,
+                       void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const uint8_t * src1 = _src1;
+  const uint8_t * src2 = _src2;
+  uint8_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) - *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-128,127) + 128;
+    }
+  }
+
+
+static void sub_u16_c(const void * _src1, const void * _src2,
+                      void * _dst, int num)
+  {
+  int32_t tmp;
+  int i = num+1;
+  const uint16_t * src1 = _src1;
+  const uint16_t * src2 = _src2;
+  uint16_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) - *(src2++);
+    *(dst++) = RECLIP_32_TO_16(tmp);
+    }
+
+  }
+
+static void sub_s16_c(const void * _src1, const void * _src2,
+                      void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const int16_t * src1 = _src1;
+  const int16_t * src2 = _src2;
+  int16_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) - *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-32768,32767);
+    }
+  }
+
+static void sub_u16_s_c(const void * _src1, const void * _src2,
+                        void * _dst, int num)
+  {
+  int tmp;
+  int i = num+1;
+  const uint16_t * src1 = _src1;
+  const uint16_t * src2 = _src2;
+  uint16_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) + *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-32768,32767) + 32768;
+    }
+  }
+
+static void sub_s32_c(const void * _src1, const void * _src2,
+                      void * _dst, int num)
+  {
+  int64_t tmp;
+  int i = num+1;
+  const int32_t * src1 = _src1;
+  const int32_t * src2 = _src2;
+  int32_t * dst  = _dst;
+
+  while(--i)
+    {
+    tmp = *(src1++) - *(src2++);
+    *(dst++) = GENERIC_CLIP(tmp,-2147483648,2147483647);
+    }
+  }
+
+static void sub_float_c(const void * _src1, const void * _src2,
+                        void * _dst, int num)
+  {
+  int i = num+1;
+  const float * src1 = _src1;
+  const float * src2 = _src2;
+  float * dst  = _dst;
+
+  while(--i)
+    {
+    *(dst++) = *(src1++) - *(src2++);
+    }
+  }
+
+static void sub_double_c(const void * _src1, const void * _src2,
+                         void * _dst, int num)
+  {
+  int i = num+1;
+  const double * src1 = _src1;
+  const double * src2 = _src2;
+  double * dst  = _dst;
+  
+  while(--i)
+    {
+    *(dst++) = *(src1++) - *(src2++);
+    }
+  }
 
 
 void gavl_dsp_init_c(gavl_dsp_funcs_t * funcs, 
@@ -451,5 +740,25 @@ void gavl_dsp_init_c(gavl_dsp_funcs_t * funcs,
   funcs->bswap_16          = bswap_16_c;
   funcs->bswap_32          = bswap_32_c;
   funcs->bswap_64          = bswap_64_c;
-  
+
+  funcs->add_u8            = add_u8_c;
+  funcs->add_s8            = add_s8_c;
+  funcs->add_u8_s          = add_u8_s_c;
+  funcs->add_u16           = add_u16_c;
+  funcs->add_s16           = add_s16_c;
+  funcs->add_u16_s         = add_u16_s_c;
+  funcs->add_s32           = add_s32_c;
+  funcs->add_float         = add_float_c;
+  funcs->add_double        = add_double_c;
+
+  funcs->sub_u8            = sub_u8_c;
+  funcs->sub_s8            = sub_s8_c;
+  funcs->sub_u8_s          = sub_u8_s_c;
+  funcs->sub_u16           = sub_u16_c;
+  funcs->sub_s16           = sub_s16_c;
+  funcs->sub_u16_s         = sub_u16_s_c;
+  funcs->sub_s32           = sub_s32_c;
+  funcs->sub_float         = sub_float_c;
+  funcs->sub_double        = sub_double_c;
+
   }
