@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -34,18 +34,24 @@ static void (FUNC_NAME)(gavl_transform_context_t * ctx, gavl_transform_pixel_t *
   * src_3,
 #endif
   *dst;
-    
+
+  gavl_transform_pixel_t * pixel;
+  
 #ifdef INIT
   INIT
 #endif
+
+  pixel = pixels;    
+
+  i = ctx->dst_width+1;
   
-  for(i = 0; i < ctx->dst_width; i++)
+  while(--i)
     {
-    if(!pixels[i].outside)
+    if(!pixel->outside)
       {
       dst = (TYPE*)(dest_start);
-      src_0 = (TYPE*)(ctx->src + ctx->advance * pixels[i].index_x +
-                      ctx->src_stride * pixels[i].index_y);
+      src_0 = (TYPE*)(ctx->src + ctx->advance * pixel->index_x +
+                      ctx->src_stride * pixel->index_y);
 #if NUM_TAPS > 1
       src_1 = (TYPE*)((uint8_t*)(src_0) + ctx->src_stride);
 #endif
@@ -58,13 +64,21 @@ static void (FUNC_NAME)(gavl_transform_context_t * ctx, gavl_transform_pixel_t *
       TRANSFORM
       }
     dest_start += ctx->advance;
+    pixel++;
     }
+#ifdef FINISH
+  FINISH
+#endif
   }
 
 #ifdef INIT
 #undef INIT
 #endif
 
+#ifdef FINISH
+#undef FINISH
+#endif
+
 #undef FUNC_NAME
 #undef TYPE
 #undef TRANSFORM
