@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -23,6 +23,8 @@
 #include <video.h>
 #include <transform.h>
 #include <stdio.h>
+#include <accel.h>
+
 
 static gavl_transform_scanline_func get_func(gavl_transform_funcs_t * tab,
                                              gavl_pixelformat_t pixelformat,
@@ -57,11 +59,11 @@ static gavl_transform_scanline_func get_func(gavl_transform_funcs_t * tab,
       return tab->transform_uint8_x_2;
       break;
     case GAVL_GRAY_16:
-      *bits = tab->bits_uint16;
+      *bits = tab->bits_uint16_x_1;
       return tab->transform_uint16_x_1;
       break;
     case GAVL_GRAYA_32:
-      *bits = tab->bits_uint16;
+      *bits = tab->bits_uint16_x_2;
       return tab->transform_uint16_x_2;
       break;
     case GAVL_YUVA_32:
@@ -91,16 +93,16 @@ static gavl_transform_scanline_func get_func(gavl_transform_funcs_t * tab,
       break;
     case GAVL_YUV_444_P_16:
     case GAVL_YUV_422_P_16:
-      *bits = tab->bits_uint16;
+      *bits = tab->bits_uint16_x_1;
       return tab->transform_uint16_x_1;
       break;
     case GAVL_RGB_48:
-      *bits = tab->bits_uint16;
+      *bits = tab->bits_uint16_x_3;
       return tab->transform_uint16_x_3;
       break;
     case GAVL_RGBA_64:
     case GAVL_YUVA_64:
-      *bits = tab->bits_uint16;
+      *bits = tab->bits_uint16_x_4;
       return tab->transform_uint16_x_4;
       break;
     case GAVL_GRAY_FLOAT:
@@ -130,19 +132,45 @@ static void init_func_tab(gavl_video_options_t * opt,
                           gavl_transform_context_t * ctx,
                           gavl_transform_funcs_t * func_tab)
   {
+  if(!opt->quality)
+    memset(func_tab, 0, sizeof(*func_tab));
+  
   switch(ctx->tab.factors_per_pixel)
     {
     case 1:
-      gavl_init_transform_funcs_nearest_c(func_tab, ctx->advance);
+      if((opt->quality > 0) || (opt->accel_flags & GAVL_ACCEL_C))
+        gavl_init_transform_funcs_nearest_c(func_tab, ctx->advance);
       break;
     case 2:
-      gavl_init_transform_funcs_bilinear_c(func_tab, ctx->advance);
+      if((opt->quality > 0) || (opt->accel_flags & GAVL_ACCEL_C))
+        gavl_init_transform_funcs_bilinear_c(func_tab, ctx->advance);
+#ifdef HAVE_MMX
+      if((opt->quality < 3) && (opt->accel_flags & GAVL_ACCEL_MMX))
+        gavl_init_transform_funcs_bilinear_mmx(func_tab, ctx->advance);
+      if((opt->quality < 3) && (opt->accel_flags & GAVL_ACCEL_MMXEXT))
+        gavl_init_transform_funcs_bilinear_mmxext(func_tab, ctx->advance);
+      
+#endif
       break;
     case 3:
-      gavl_init_transform_funcs_quadratic_c(func_tab, ctx->advance);
+      if((opt->quality > 0) || (opt->accel_flags & GAVL_ACCEL_C))
+        gavl_init_transform_funcs_quadratic_c(func_tab, ctx->advance);
+#ifdef HAVE_MMX
+      if((opt->quality < 3) && (opt->accel_flags & GAVL_ACCEL_MMX))
+        gavl_init_transform_funcs_quadratic_mmx(func_tab, ctx->advance);
+      if((opt->quality < 3) && (opt->accel_flags & GAVL_ACCEL_MMXEXT))
+        gavl_init_transform_funcs_quadratic_mmxext(func_tab, ctx->advance);
+#endif
       break;
     case 4:
-      gavl_init_transform_funcs_bicubic_c(func_tab, ctx->advance);
+      if((opt->quality > 0) || (opt->accel_flags & GAVL_ACCEL_C))
+        gavl_init_transform_funcs_bicubic_c(func_tab, ctx->advance);
+#ifdef HAVE_MMX
+      if((opt->quality < 3) && (opt->accel_flags & GAVL_ACCEL_MMX))
+        gavl_init_transform_funcs_bicubic_mmx(func_tab, ctx->advance);
+      if((opt->quality < 3) && (opt->accel_flags & GAVL_ACCEL_MMXEXT))
+        gavl_init_transform_funcs_bicubic_mmxext(func_tab, ctx->advance);
+#endif
       break;
     default:
       fprintf(stderr, "BUG: Filter taps > 4 in image transform\n");
@@ -150,7 +178,7 @@ static void init_func_tab(gavl_video_options_t * opt,
     }
   }
 
-void
+int
 gavl_transform_context_init(gavl_image_transform_t * t,
                             gavl_video_options_t * opt,
                             int field_index, int plane_index,
@@ -251,11 +279,14 @@ gavl_transform_context_init(gavl_image_transform_t * t,
                        t->format.pixelformat,
                        &bits);
 
+  if(!ctx->func)
+    return 0;
+  
   /* Now we know the bits, convert to int */
   if(bits)
     gavl_transform_table_init_int(&ctx->tab,
                                   bits, ctx->dst_width, ctx->dst_height);
-  
+  return 1;
   }
 
 static void func_1(void* p, int start, int end)
@@ -278,9 +309,12 @@ static void func_1(void* p, int start, int end)
     ctx->func(ctx, ctx->tab.pixels[i], dst_save);
     dst_save += dst_stride;
     }
-  // #ifdef HAVE_MMX
-  //  __asm__ __volatile__ ("emms");
-  // #endif
+#ifdef HAVE_MMX
+  if(ctx->need_emms)
+    {
+    __asm__ __volatile__ ("emms");
+    }
+#endif
   }
 
 
@@ -340,7 +374,13 @@ void gavl_transform_context_transform(gavl_transform_context_t * ctx,
       ctx->func(ctx, ctx->tab.pixels[i], dst_save);
       dst_save += dst_stride;
       }
-
+#ifdef HAVE_MMX
+    if(ctx->need_emms)
+      {
+      __asm__ __volatile__ ("emms");
+      // fprintf(stderr, "emms");
+      }
+#endif
     }
   
   }
