@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -255,6 +255,189 @@ typedef struct
    */
   void (*bswap_64)(void * ptr, int len);
   
+  /** \brief Add 2 uint8_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  
+  void (*add_u8)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Add 2 uint8_t vectors (for audio stored as unsigned)
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_u8_s)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Add 2 int8_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_s8)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Add 2 uint16_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_u16)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Add 2 uint16_t vectors (for audio stored as unsigned)
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_u16_s)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Add 2 int16_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_s16)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Add 2 int32_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_s32)(const void * src1, const void * src2, void * dst, int num);
+  
+  /** \brief Add 2 float vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_float)(const void * src1, const void * src2, void * dst, int num);
+  
+  /** \brief Add 2 float vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*add_double)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Subtract 2 uint8_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  
+  void (*sub_u8)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Subtract 2 uint8_t vectors (for audio stored as unsigned)
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_u8_s)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Subtract 2 int8_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_s8)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Subtract 2 uint16_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_u16)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Subtract 2 uint16_t vectors (for audio stored as unsigned)
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_u16_s)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Subtract 2 int16_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_s16)(const void * src1, const void * src2, void * dst, int num);
+
+  /** \brief Subtract 2 int32_t vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_s32)(const void * src1, const void * src2, void * dst, int num);
+  
+  /** \brief Subtract 2 float vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_float)(const void * src1, const void * src2, void * dst, int num);
+  
+  /** \brief Subtract 2 float vectors
+   *  \param src1 Source 1
+   *  \param src2 Source 2
+   *  \param dst  Destination
+   *  \param num  Number of entries
+   *
+   *  src1 and/or src2 can be identical to dst
+   */
+  void (*sub_double)(const void * src1, const void * src2, void * dst, int num);
+
+  
   } gavl_dsp_funcs_t;
 
 /** \brief Create a DSP context
