@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -26,6 +26,7 @@
 extern "C" {
 #endif
 
+#include <inttypes.h>
 #include <gavl/gavldefs.h>
 
 /*! \defgroup time Time
@@ -262,7 +263,7 @@ void gavl_timer_set(gavl_timer_t * timer, gavl_time_t t);
 
 /*! \ingroup timer
  * \brief Get the current time for benchmarking
- * \param flags Flags returned by \reg gavl_accel_supported
+ * \param flags Flags returned by \ref gavl_accel_supported
  * \returns The time in arbitrary units
  *
  * The returned value itself is meaningless since the
@@ -276,7 +277,7 @@ uint64_t gavl_benchmark_get_time(int flags);
 
 /*! \ingroup timer
  * \brief Get a description about the value returned by \ref gavl_benchmark_get_time
- * \param flags Flags returned by \reg gavl_accel_supported
+ * \param flags Flags returned by \ref gavl_accel_supported
  * \returns A string describing what the time value means
  *
  */
