@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -47,7 +47,7 @@ typedef uint64_t gavl_timecode_t; //!< Typedef for timecodes
 
 #define GAVL_TIMECODE_DROP_FRAME (1<<0) //!< NTSC drop frame is used
 
-#define GAVL_TIMECODE_UNDEFINED GAVL_TIMECODE_INVALID_MASK //! Convenience macro
+#define GAVL_TIMECODE_UNDEFINED GAVL_TIMECODE_INVALID_MASK //!< Convenience macro
   
 /** \brief Timecode format
  *
@@ -61,6 +61,17 @@ typedef struct
   int int_framerate; //!< Integer framerate. A value of zero signals, that no timecodes are available.
   int flags;         //!< Zero or more of the flags defined above
   } gavl_timecode_format_t;
+
+/*!
+  \brief Copy one timecode format to another
+  \param dst Destination format 
+  \param src Source format 
+ */
+  
+GAVL_PUBLIC
+void gavl_timecode_format_copy(gavl_timecode_format_t * dst,
+                               const gavl_timecode_format_t * src);
+
   
 /** \brief Extract the time part of the timecode
  *  \param tc A timecode
@@ -120,7 +131,6 @@ void gavl_timecode_from_ymd(gavl_timecode_t * tc,
 
 /** \brief Get the frame count from the timecode
  *  \param tf The timecode format
- *  \param vf The video format
  *  \param tc A timecode
  *  \returns The frame count
  *
@@ -134,7 +144,6 @@ int64_t gavl_timecode_to_framecount(const gavl_timecode_format_t * tf,
 
 /** \brief Get a timecode from the frame count
  *  \param tf The timecode format
- *  \param vf The video format
  *  \param fc The frame count
  *  \returns The timecode corresponding to the framecount
  *
