@@ -1,7 +1,7 @@
 /*****************************************************************
  * gavl - a general purpose audio/video processing library
  *
- * Copyright (c) 2001 - 2008 Members of the Gmerlin project
+ * Copyright (c) 2001 - 2010 Members of the Gmerlin project
  * gmerlin-general@lists.sourceforge.net
  * http://gmerlin.sourceforge.net
  *
@@ -61,7 +61,10 @@ typedef struct
   int bits_rgb_16;
   int bits_uint8_noadvance;
   int bits_uint8_advance;
-  int bits_uint16;
+  int bits_uint16_x_1;
+  int bits_uint16_x_2;
+  int bits_uint16_x_3;
+  int bits_uint16_x_4;
   } gavl_transform_funcs_t;
 
 void gavl_init_transform_funcs_nearest_c(gavl_transform_funcs_t * tab,
@@ -73,6 +76,24 @@ void gavl_init_transform_funcs_quadratic_c(gavl_transform_funcs_t * tab,
 void gavl_init_transform_funcs_bicubic_c(gavl_transform_funcs_t * tab,
                                          int advance);
 
+#ifdef HAVE_MMX
+void gavl_init_transform_funcs_bilinear_mmx(gavl_transform_funcs_t * tab,
+                                            int advance);
+void gavl_init_transform_funcs_bilinear_mmxext(gavl_transform_funcs_t * tab,
+                                               int advance);
+
+void gavl_init_transform_funcs_quadratic_mmx(gavl_transform_funcs_t * tab,
+                                             int advance);
+void gavl_init_transform_funcs_quadratic_mmxext(gavl_transform_funcs_t * tab,
+                                                int advance);
+
+void gavl_init_transform_funcs_bicubic_mmx(gavl_transform_funcs_t * tab,
+                                           int advance);
+void gavl_init_transform_funcs_bicubic_mmxext(gavl_transform_funcs_t * tab,
+                                              int advance);
+
+#endif
+
 
 
 typedef struct 
@@ -115,9 +136,13 @@ struct gavl_transform_context_s
   gavl_video_options_t * opt;
 
   gavl_video_frame_t * dst_frame;
+#ifdef HAVE_MMX
+  int need_emms;
+#endif
+
   };
 
-void
+int
 gavl_transform_context_init(gavl_image_transform_t * t,
                             gavl_video_options_t * opt,
                             int field_index, int plane_index,
